/* Copyright (c) 1988 Carrick Sean Casey. All rights reserved. */

/* main program loop */
/* active clients, new connections, and perform asynch dungeon action */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>
#ifdef FCNTL
#include <fcntl.h>
#endif FCNTL
#include "select.h"
#include "ipcf.h"
#include "externs.h"
#include "../server/server.h"

/* Stolen from Solaris land. I don't know why (unsigned) matters, but it does */
#ifdef sun
#ifdef FD_ISSET
#undef FD_ISSET
#define FD_ISSET(n, p)  ((p)->fds_bits[(n)/NFDBITS] & ((unsigned) 1 << ((n) % NFDBITS)))
#endif
#endif 

extern int errno;
extern int restart;
fd_set fdr, efdr;
int timeisup;

serverserve()
{
	int ret, i;
	int gotalarm();
	long loopcount = 0; /* used to break loop for periodic didpoll */
	char tbuf[256];

	/* look at fd where new clients connect */
	if (restart == 0) {
		FD_ZERO(&fdset);
		FD_SET(port_fd, &fdset);
		}
	else 
		for (i = 0; i < MAX_USERS+1; i++)
			cbufs[i].new = 1;

	if (port_fd > highestfd)
		highestfd = port_fd;

	signal(SIGALRM, gotalarm);
	timeisup = 0;

	for (;;) {
		if (polldelay) {
			if (setitimer(ITIMER_REAL, polldelay, 0) < 0) {
				perror("server: setitimer failed");
				exit(-1);
			}
			timeisup = 0;
/*			sigpause(0); */
		}


		bcopy((char *) &fdset, (char *) &fdr,
		  (unsigned int) sizeof(fdset));
		bcopy((char *) &fdset, (char *) &efdr,
		  (unsigned int) sizeof(fdset));

		/* wait for input */
		/* NULL should be changed to a time value that reflects */
		/* how much activity is going on */

		setpolltimeout(10000);

		if ((ret = select(highestfd+1, &fdr, 0, &efdr, polltimeout)) < 1) {
			if (ret == 0 || errno == EINTR) {
				s_didpoll();
				timeisup = 0;
			} else {
				perror("select");
				/* exit(-1); */
			}
		}

		if (ret > 0)
			sdoinput();
		if (timeisup || !(++loopcount % 10))
			s_didpoll();
	}
}

gotalarm()
{
	timeisup++;
}

sdoinput()
{
	int n;
	register int x;
	register unsigned long bits;

	for (x = 1; x <= highestfd; x++)
		if (FD_ISSET(x, &efdr) > 0) {
			disconnectuser(x);
			}


	/* examine set of file descriptors */
	for (x = 1; x <= highestfd; x++)
		if (FD_ISSET(x, &fdr) > 0) {
			if (x == port_fd) {
				/* new connect on advertised socket */
				if ((n = _newconnect(x)) > 0)
					/* let server init the user */
					s_new_user(n);
			} else {
				/* fd is client input */
				switch(_readpacket(x, &cbufs[x])) {
				case  1:
					/* complete packet */
					s_packet(x, cbufs[x].buf);
					break;
				case  0:
					/* incomplete packet */
					break;
				case -1:
					/* error */
					break;
				case -2:
					/* close connection */
					disconnectuser(x);
					break;
				}
			}
		}
}

