/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

#include "server.h"
#include "config.h"
#include "externs.h"
#include "groups.h"
#include "protocol.h"
#include "version.h"
#include <sys/wait.h>
#include <sys/file.h>
#include <sys/dir.h>
#include <signal.h>
#include <strings.h>
#include <sys/time.h>
#include <sys/resource.h>

main (argc, argv)
int argc;
char **argv;
{
	int c;
	int ret;
	extern char *optarg;
	char *getenv();
	int clearargsflg = 0;
	int forkflag = 0;
	int port = DEFAULTPORT;
	int i;
	long time();
	long TheTime;
	int pidf;
	char tbuf[255];
	struct rlimit rlp;

	restart = 0;

	setbuf(stdout, (char *) 0);

	while ((c = getopt(argc, argv, "cp:fR")) != EOF) {

		switch (c) {

		case 'R':
			restart++;
			break;

		case 'f':
			forkflag++;
			break;

		case 'c':
			clearargsflg++;
			break;

		case 'p':
			port = atoi(optarg);
			break;

		case '?':
		default:
			puts("usage: icbd [-cs string] \r");
			puts("-c	 wipe args from command line\r");
			exit(-1);
			break;
		}
	}

#ifdef RLIMIT_NOFILE
   getrlimit(RLIMIT_NOFILE, &rlp);
   rlp.rlim_cur = MAX_USERS + 1;
   if (setrlimit(RLIMIT_NOFILE, &rlp) < 0) {
	perror("setrlimit");
	exit(1);
	}
#endif

   if (restart == 0) {

	open_logs();

	if (makenewport(thishost, port) < 0) {
		sprintf(tbuf, "[ERROR] makenewport failed: %s", 
			sys_errlist[errno]);
		mdb(tbuf);
		exit (-1);
	}

	if (forkflag == 0)
		if (fork()) exit(0);

	if (clearargsflg)
		clearargs(argc, argv);

	/* get our hostname */
	if ((ret=gethostname(thishost,MAXHOSTNAMELEN)) < 0) {
		sprintf(tbuf, "[ERROR] gethostname failed: %s", 
			sys_errlist[errno]);
		mdb(tbuf);
		}

	if (strcmp(thishost, SHORT_HOSTNAME) == 0)
		strcpy(thishost, FQDN);

	/* initialize out tables  */
	init_groups();
	clear_users();

	/* invent our resident automagical user */
	TheTime = time(NULL);
	fill_user_entry(NICKSERV, "server",thishost,
		"Server", "", "ICB", TheTime, 0);
	u_tab[NICKSERV].login = 1;
	u_tab[NICKSERV].t_on = TheTime;
	u_tab[NICKSERV].t_recv = TheTime;
	strcpy(u_tab[NICKSERV].realname, "registered");
	fill_group_entry(0, "ICB", "...here to serve you!", SUPERSECRET,
		RESTRICTED, NICKSERV, QUIET);
	nickwritetime(NICKSERV, 0);

	sprintf(mbuf, "[INFO] ICB revision %s.", VERSION);
	mdb(mbuf);
	sprintf(mbuf, "[INFO] There are %d users possible.", MAX_USERS);
	mdb(mbuf);
	sprintf(mbuf, "[INFO] Of those, %d are real.", MAX_REAL_USERS);
	mdb(mbuf);

	for (i = 0; i < MAX_REAL_USERS; i++)
		S_kill[i] = 0;
	
	setpolldelay(POLL_DELAY);

	setpolltimeout(IDLE_POLLWAIT);

	trapsignals();
   }
   else {
	open_logs();
	/* get our hostname */
	if ((ret=gethostname(thishost,MAXHOSTNAMELEN)) < 0) {
		sprintf(tbuf, "[ERROR] gethostname failed: %s", 
			sys_errlist[errno]);
		mdb(tbuf);
		}

	if (strcmp(thishost, SHORT_HOSTNAME) == 0)
		strcpy(thishost, FQDN);

	setpolldelay(POLL_DELAY);
	setpolltimeout(IDLE_POLLWAIT);
	init_groups();
	clear_users();
	trapsignals(); 
	mdb("[RESTART] Reloading server data");
	icbload();
	for (i = 0; i < MAX_USERS; i++)
		if (u_tab[i].login == 1)
			sendimport(i, "Restart", "Server is back up.");

	}

        if ((pidf = open(PID_FILE, O_WRONLY|O_CREAT|O_TRUNC)) >= 0)
        {
                /* write the process id on line 1 */
                sprintf(mbuf, "%d\n", getpid());
		write(pidf, mbuf, strlen(mbuf));
		close(pidf);
        }

#ifdef RLIMIT_NOFILE
        getrlimit(RLIMIT_NOFILE, &rlp);
        sprintf(tbuf, "[INFO] Max fds set to %d", rlp.rlim_cur);
        mdb(tbuf);
#endif

	/* start the serve loop */
	if (serverserve() < 0) {
		sprintf(tbuf, "[ERROR] serverserve failed: %s", 
			sys_errlist[errno]);
		mdb(tbuf);
		exit(-1);
	}

	/* all's well that ends */
	exit(0);
}

open_logs()
{
	if((icbd_log = open(ICBDLOG,O_WRONLY|O_SYNC|O_CREAT|O_APPEND)) < 0) {
		fprintf(stderr, "icbd: could not open log file.\n");
		perror("ICBDLOG open");
		exit(1);
		}
}

close_logs()
{
	close(icbd_log);
}

cycle_logs()
{
	(void) close_logs();
	(void) open_logs();
}

icbexit()
{
	int user;

	mdb("someone told us to exit");
	for (user=0; user < MAX_REAL_USERS; user++) {
		if (u_tab[user].login == 1) {
			disconnectuser(user);
		}
	}
	exit(0);
}
