/* Copyright (c) 1991 by John Atwood deVries II. */

/* handle various messages from the client */

#include "server.h"
#include "externs.h"
#include "lookup.h"
#include "groups.h"
#include <strings.h>
#include <errno.h>
#include <fcntl.h>

/* open message */
/*
   note that we need to know who sent the message.  it doesn't
   come with the sender's nick, so we have to reformat the message
   before sending it to the group
*/

openmsg(n, pkt)
int n;
char *pkt;
{
	long TheTime;
	int gi;

        if (u_tab[n].login == 1) {
	        /* record the time */
        	TheTime = time(NULL);
                u_tab[n].t_recv= TheTime;
		
		if (split(pkt) != 1) {
			mdb("[ERROR] got bad open message packet");
		} else {
#ifdef VERBOSE
		   sprintf(mbuf,"[OPEN] %d", n);
		   mdb(mbuf);
#endif
		   gi = find_group(u_tab[n].group);
		   if (g_tab[gi].volume == QUIET)
			senderror(n,
			"Open messages not permitted in quiet groups.");
		   else if (count_users_in_group(g_tab[gi].name) < 2)
			senderror(n,
			"No one else in group!");
		   else {
			s_send_group(n);
		   }
		}
	} else {
		mdb("[ERROR] cannot send open messages until logged in");
	}
}


/*
   they sent us a login message. we need to put their info into 
   the user information table and then send them back a loginok message.
   for any field they didn't specify, come up with a plausible default.
   we also need to handle "w" vs. "login"
   we also need to dump them if they try using a:
	nickname already in use -or-
	group they aren't allowed to enter.
   their loginid may NOT be blank or null
*/

loginmsg(n, pkt)
int n;		/* socket on which they sent message */
char *pkt;	/* packet buffer */
{
	int num_fields;
	char which_group[MAX_NICKLEN+4];
	char temp[MAX_NICKLEN+4];
	char one[255], two[255], three[255];
	int len;
	int ret;
	int how_many;
	int i, j;
	long TheTime;
	int gi;
	int target_user;
	char * cp;
	extern char * getremotename();
	int access_file;
	char c;

	if (u_tab[n].login == 1) {
		senderror(n, "Already logged in.");
		return 1;
		}

	u_tab[n].login = 0;
	/* just in case */
	TheTime = time(NULL);

	num_fields = split(pkt);

	if (num_fields < 4) return -1;

	len = strlen(fields[2]);

	bzero(temp, MAX_NICKLEN+4);
	bzero(which_group, MAX_NICKLEN+4);

	if (len == 0) { /* no group specified, give them one */
		strcpy(which_group,"1");
	} else {
	 	how_many = (len > (MAX_NICKLEN+3))? (MAX_NICKLEN+3): len;
		strncpy(which_group, fields[2], how_many);
		}

	if ( strcasecmp(fields[3],"w") == 0) {
		/* who only */
		fields[0][0] = 'w';
		fields[0][1] = '\000';
		fields[1][0] = '\000';
		s_who(n, 2);
		return -1; /* fail */
	} else {
	/* regular login */

	/* make sure the nickname hasn't already been taken */
	if (find_user(fields[1]) >= 0) {
		/* oops, someone already has this nick */
		senderror(n,"Nickname already in use.");
		return(-1);
	}

	/* make sure it isn't a null nickname */
	if ((strlen(fields[1]) <= 0) || (strlen(fields[1]) > MAX_NICKLEN)) {
		sprintf(mbuf, "Nickname must be between 1 and %d characters.",
			MAX_NICKLEN);
		senderror(n, mbuf);
		return -1;
	}

	/* make sure the length of the loginid isn't wrong */
	if ((strlen(fields[0]) <= 0) || (strlen(fields[0]) > MAX_IDLEN)) {
		sprintf(mbuf, "Login id must be between 1 and %d characters.",
			MAX_IDLEN);
		senderror(n, mbuf);
		return -1;
	}
	for (i = 0; i < strlen(fields[0]); i++)
		if (isalnum(fields[0][i]) == 0) {
			senderror(n, "Login id must contain only alphanumeric characters.");
			return -1;
		}

	/* This is set up in s_new_user now */
	cp = u_tab[n].nodeid;

	bzero(one, 255);
	bzero(three, 255);
        sprintf(one, "%s@%s", fields[0], cp);
	ucaseit(one);
	access_file = open(ACCESS_FILE, O_RDONLY);
	if (access_file >= 0) {
	   while ((i = read(access_file, &c, 1)) > 0) {
		if (isspace(c)) {
			ucaseit(three);
			if (wildmat(one, three)) {
			   sprintf (mbuf, "Login refused for %s", one);
			   senderror(n, mbuf);
			   mdb (mbuf);
			   bzero(two, 255);
			   while ((i = read(access_file, &c, 1)) > 0) {
			      if (c == '\012') break;
			      strncat(two, &c, 1);
			      }
			   sprintf (mbuf, "Reason: %s", two);
			   senderror(n, mbuf);
			   sprintf (mbuf, "Email %s for details.", SERVER_ADMIN);
			   senderror(n, mbuf);
	   		   if (close(access_file) != 0) {
	   			sprintf(mbuf, "[ERROR] Access File Close: %s", sys_errlist[errno]);
	   			mdb(mbuf);
				}
			   return -1;
			}
			bzero(three, 255);
		}
		else {
			strncat(three, &c, 1);
		}
	   }
	   if (close(access_file) != 0) {
	   		sprintf(mbuf, "[ERROR] Access File Close: %s", sys_errlist[errno]);
	   		mdb(mbuf);
			}
	   }
	else {
	   sprintf(mbuf, "[ERROR] Access File Open: %s", sys_errlist[errno]);
	   mdb(mbuf);
	   }
		

/* get rid of nasty characters in the nickname */
/* mdb("filtering nickname"); */
	filternickname(fields[1]);
	

	/* fill in what we can */
/* mdb("filling in user entry"); */
	fill_user_entry(n, fields[0], cp, fields[1],
	    fields[4], "", 0, 0);

	sprintf(mbuf, "[LOGIN] %d: %s@%s", n, fields[0], cp);
	mdb(mbuf);

        /* make sure they are allowed to use this nickname */
	if (strcasecmp(fields[1],"admin") == 0) {
	if (!check_auth(n)){
		/* oops, password is wrong */
		senderror(n,"Nickname already in use.");
		return(-1);
	}
	}

	send_loginok(n);

	s_motd(n,2);

        if (which_group[0] == '@')
           if ((target_user = find_user(&which_group[1])) < 0) {
              senderror (n, "User not found or is in a secret or invisible group.");
              return(-1);
              }
           else
              strcpy (which_group, u_tab[target_user].group);

	/* fake a s_change */
	strcpy(fields[1],which_group);
	/* mdb(fields[1]); */
	if (s_change(n,2) < 0) {
		/* login fails because can't get into that group */
		return(-1);
	}

	/* could find group slot, all is ok */
	/* fill in last members of table entry */
	/* strcpy(u_tab[n].group, which_group); */
	u_tab[n].login = 1;
	u_tab[n].t_on = TheTime;
	u_tab[n].t_recv = TheTime;
	u_tab[n].t_sent = TheTime;
	u_tab[n].pri_n_hushed = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].pri_n_hushed, MAX_HUSHED);
	u_tab[n].pub_n_hushed = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].pub_n_hushed, MAX_HUSHED);
	u_tab[n].pri_s_hushed = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].pri_s_hushed, MAX_HUSHED);
	u_tab[n].pub_s_hushed = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].pub_s_hushed, MAX_HUSHED);
	u_tab[n].n_notifies = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].n_notifies, MAX_NOTIFIES);
	u_tab[n].s_notifies = (NAMLIST *) malloc(sizeof(NAMLIST));
	nlinit(u_tab[n].s_notifies, MAX_NOTIFIES);

	sprintf(mbuf, "Welcome to ICB %s", u_tab[n].nickname);
	sends_cmdout(n, mbuf);

        bzero(one, 255);
        bzero(two, 255);
        bzero(three, 255);
        sprintf(one, "%s@%s", u_tab[n].loginid, u_tab[n].nodeid);
        ucaseit(one);
        strcpy(three, u_tab[n].nickname);
        ucaseit(three);
        for (j = 0; j < MAX_USERS; j++)
                if ((n != j) && (u_tab[j].login > 0)) {
                        if ((nlmatch(three, *u_tab[j].n_notifies) ||
                           nlmatch(one, *u_tab[j].s_notifies)) && 
			   (g_tab[find_group(u_tab[n].group)].visibility !=
			      SUPERSECRET)) {
                                sprintf (two, "%s (%s) has just signed on",
                                        u_tab[n].nickname, one);
                                sendstatus(j, "Notify-On", two);
                                }
                }

	/* check to see if we know this person */
/*	mdb("About to do nicklookup"); */
	ret = nicklookup(-n, u_tab[n].nickname);
	if (ret == 0) {
		/* we know this person */
		strcpy(u_tab[n].realname, "registered");
		sends_cmdout(n, "Nick registered");
		nickwritetime(n, 0);
		for (i = 1; i <= MAX_GROUPS; i++)
			if ((g_tab[i].modtimeout > 0.0) &&
			  (strcmp(g_tab[i].missingmod, u_tab[n].nickname)==0)){
			  g_tab[i].modtimeout = 0;
			  g_tab[i].mod = n;
			  bzero(g_tab[i].missingmod, MAX_NICKLEN);
			  sprintf(mbuf, "%s is the active moderator again.",
				u_tab[n].nickname);
			  for (j = 1; j <= MAX_REAL_USERS; j++)
				if ((strcasecmp(u_tab[j].group, g_tab[i].name)
				    == 0) && (j != n))
				    sendstatus(j, "Mod", mbuf);
			  sprintf(mbuf, "You are the moderator of group %s",
				g_tab[i].name);
			  sendstatus(n, "Mod", mbuf);
			}
		if ((i = nickckmsg(n)) > 0)
			if (i == 1)
				sendstatus(n, "Message", "You have 1 message");
			else {
				sprintf(mbuf, "You have %d messages", i);
				sendstatus(n, "Message", mbuf);
				}
		}
	else if (ret == -2) /* we know this person but they're not validated */
	{
	sendstatus(n, "Register", "Send password to authenticate your nickname.");
	j = 0;
	for (i = 0; i < MAX_GROUPS; i++)
	    if (strcasecmp(u_tab[n].nickname, g_tab[i].missingmod) == 0) {
		sprintf(mbuf, "You are moderator of group %s", g_tab[i].name);
		sendstatus(n, "Mod", mbuf);
		j++;
	    }
	if (j == 1) sendstatus(n, "Mod", "You must register using /p <password> to regain mod of the above group.");
	if (j > 1) sendstatus(n, "Mod", "You must register using /p <password> to regain mod of the above groups.");
	}
	else {
		sendstatus(n, "No-Pass", "Your nickname does not have a password.");
		sendstatus(n, "No-Pass", "For help type /m server ?");
	}

	bzero(one, 255);
	sprintf(one, "%s@%s", u_tab[n].loginid, u_tab[n].nodeid);
	ucaseit(one);
        for (i = 0; i < MAX_GROUPS; i++)
	    if (
	       (nlpresent(u_tab[n].nickname, *g_tab[i].n_invites) > 0) ||
	       (nlmatch(one, *g_tab[i].s_invites)) ||
	       (nlpresent(u_tab[n].nickname, *g_tab[i].nr_invites) &&
		  (strlen(u_tab[n].realname) > 0)) ||
	       (nlmatch(one, *g_tab[i].sr_invites) && 
		  (strlen(u_tab[n].realname) > 0))) {
               sprintf (mbuf, "Invited to: %s", g_tab[i].name);
               sends_cmdout(n, mbuf);
	       }

	return 0;
	}
}

/* command message */

cmdmsg(n, pkt)
int n;
char *pkt;
{
	int argc;
	long TheTime;

        if (u_tab[n].login == 1) {

                /* record the time */
                TheTime = time(NULL);
                u_tab[n].t_recv= TheTime;

		argc = split(pkt);

#ifdef VERBOSE
		if (strcmp(fields[0], "m") == 0)
		  sprintf(mbuf, "[COMMAND] %d: %s %s", n, fields[0],
			getword(fields[1]));
		else
		  sprintf(mbuf, "[COMMAND] %d: %s %s", n, fields[0], fields[1]);
		mdb(mbuf);
#endif
	
		switch(lookup(fields[0], command_table)) {

			case CMD_DROP:
				s_drop(n, argc);
				break;

			case CMD_RESTART:
				s_restart(n, argc);
				break;
			
			case CMD_SHUTDOWN:
				s_shutdown(n, argc);
				break;

			case CMD_WALL:
				s_wall(n, argc);
				break;

		 	case CMD_BEEP:
				s_beep(n, argc);
				break;

			case CMD_CANCEL:
				s_cancel(n, argc);
				break;

			case CMD_G:	
				s_change(n, argc);
				break;

			case CMD_INVITE:
				s_invite(n, argc);
				break;

			case CMD_PASS:
				s_pass(n, argc);
				break;

			case CMD_BOOT:
				s_boot(n, argc);
				break;

			case CMD_STATUS:
				s_status(n, argc);
				break;

			case CMD_TOPIC:
				s_topic(n, argc);
				break;

			case CMD_MOTD:
				s_motd(n, argc);
				break;

			case CMD_M:
				s_personal(n, argc);
				break;

			case CMD_ECHOBACK:
				s_echoback(n, argc);
				break;

			case CMD_NAME:
				s_name(n, argc);
				break;

			case CMD_V:
				s_version(n, argc);
				break;

			case CMD_W:
				s_who(n, argc);
				break;

			case CMD_INFO:
				s_info(n, argc);
				break;

			case CMD_NEWS:
				s_news(n, argc);
				break;

			case CMD_HUSH:
			case CMD_SHUSH:
				s_hush(n, argc);
				break;

			case CMD_HELP:
				s_help(n, argc);
				break;

			case CMD_EXCLUDE:
				s_exclude(n, argc);
				break;

			case CMD_SHUTTIME:
				s_shuttime(n, argc);
				break;

			case CMD_NOTIFY:
				s_notify(n, argc);
				break;

		default:
			if (lookup(fields[0], auto_table) >= 0)
				autoCommand(n, 0);
			else {
				sendstatus(n, "Server", "Unknown command");
				sprintf(mbuf,
			 		"[INFO] %d: Invalid command \"%s\"",
					n, fields[0]);
				mdb(mbuf);
				}
		}
	} else {
		mdb("[ERROR] cannot issue command until logged in");
	}
}


/* pong message */

pong(n, pkt)
int n;
char *pkt;
{
	long TheTime;
	char tbuf[255];

        if (u_tab[n].login == 1) {
		sprintf(tbuf, "[INFO] %d: pong message", n);
        	mdb("tbuf");
                /* record the time */
                TheTime = time(NULL);
                u_tab[n].t_recv= TheTime;

	} else {
		 mdb("[ERROR] cannot send pong message until signed on");
	}
}
