/* copyright 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* primitive to change user info */

#include "config.h"
#include "server.h"
#include "externs.h"
#include <strings.h>
#include "protocol.h"
#include <errno.h>
#include <fcntl.h>

s_help(n, argc)
int n;
int argc;
{
int help_fd;
char c;
int i;
char tbuf[256];

help_fd = open(ICBDHELP,O_RDONLY);

/* if the file is there, list it, otherwise report error */
if (help_fd >= 0) {
	bzero(tbuf, 255);
	while ((i = read(help_fd, &c, 1)) > 0) {
		if (c == '\012') {
			sends_cmdout(n, tbuf);
			bzero(tbuf, 255);
			}
		else strncat(tbuf, &c, 1);
	}
	if (close(help_fd) != 0) {
		sprintf(mbuf, "[ERROR] Help File Close: %s",
			sys_errlist[errno]);
		mdb(mbuf);
		}
	}
else {
	sprintf(mbuf, "[ERROR] Help File Open: %s",
		sys_errlist[errno]);
	mdb(mbuf);
	senderror(n, "No help file found.");
	}
}

s_hush(n, argc)
int n;
int argc;
{
   int i;
   int personal = -1;
   int public = -1;
   int mode = -1;
   int quiet = -1;
   char *flags;
   char *who;

	who = fields[1];
	while (who[0] == '-') {
		flags = getword(who);
		for (i = 1; i < strlen(flags); i++)
			switch(flags[i]) {
			case 'q':
				if (quiet == -1)
					quiet = 1;
				else {
					senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 'n':
				if (mode == -1)
					mode = 0;
				else {
					senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 's':
				if (mode == -1)
					mode = 1;
				else {
					senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 'p':
				if (personal == -1)
					personal = 1;
				else {
					senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 'o':
				if (public == -1)
					public = 1;
				else {
					senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
					return -1;
					}
				break;
			default:
				senderror(n, "Usage: hush {-q} {-p} {-o} {-n nickname | -s address}");
				return -1;
				break;
			}
		who = get_tail(who);
		}
	 if (quiet == -1) quiet = 0;
	 if (mode == -1) mode = 0;
	 if ((personal == -1) && (public == -1)) {
		personal = 1;
		public = 1;
		}

      if (strlen(who) > 0) {
	ucaseit(who);
	if (mode == 1) { /* site addr */
	   if (personal == 1) {
              if (nlpresent(who, u_tab[n].pri_s_hushed) == 0) {
		 if (nlcount(*u_tab[n].pri_s_hushed) == MAX_HUSHED) {
			senderror(n, "Max number of site personal hushes reached.");
			return -1;
			}
		 nlput(u_tab[n].pri_s_hushed, who);
		 if (quiet == 0) {
		    sprintf(mbuf, "%s added to site personal hush list.", who);
		    sendstatus(n, "Hush", mbuf);
		    }
		 }
	      else {
		 nldelete(u_tab[n].pri_s_hushed, who);
		 if (quiet == 0) {
		        sprintf(mbuf, "%s removed from site personal hush list.", who);
		        sendstatus(n, "Hush", mbuf);
			}
		 }
	      }
	   if (public == 1) {
              if (nlpresent(who, u_tab[n].pub_s_hushed) == 0) {
		 if (nlcount(*u_tab[n].pub_s_hushed) == MAX_HUSHED) {
			senderror(n, "Max number of site open hushes reached.");
			return -1;
			}
		 nlput(u_tab[n].pub_s_hushed, who);
		 if (quiet == 0) {
		    sprintf(mbuf, "%s added to site open hush list.", who);
		    sendstatus(n, "Hush", mbuf);
		    }
		 }
	      else {
		 nldelete(u_tab[n].pub_s_hushed, who);
		 if (quiet == 0) {
		        sprintf(mbuf, "%s removed from site open hush list.", who);
		        sendstatus(n, "Hush", mbuf);
			}
		 }
	      }
	}
	else { /* nickname */
	   if (personal == 1) {
              if (nlpresent(who, u_tab[n].pri_n_hushed) == 0) {
		 if (nlcount(*u_tab[n].pri_n_hushed) == MAX_HUSHED) {
			senderror(n, "Max number of nickname personal hushes reached.");
			return -1;
			}
		 nlput(u_tab[n].pri_n_hushed, who);
		 if (quiet == 0) {
		    sprintf(mbuf, "%s added to nickname personal hush list.", who);
		    sendstatus(n, "Hush", mbuf);
		    }
		 }
	      else {
		 nldelete(u_tab[n].pri_n_hushed, who);
		 if (quiet == 0) {
		        sprintf(mbuf, "%s removed from nickname personal hush list.", who);
		        sendstatus(n, "Hush", mbuf);
			}
		 }
	      }
	   if (public == 1) {
              if (nlpresent(who, u_tab[n].pub_n_hushed) == 0) {
		 if (nlcount(*u_tab[n].pub_n_hushed) == MAX_HUSHED) {
			senderror(n, "Max number of nickname open hushes reached.");
			return -1;
			}
		 nlput(u_tab[n].pub_n_hushed, who);
		 if (quiet == 0) {
		    sprintf(mbuf, "%s added to nickname open hush list.", who);
		    sendstatus(n, "Hush", mbuf);
		    }
		 }
	      else {
		 nldelete(u_tab[n].pub_n_hushed, who);
		 if (quiet == 0) {
		        sprintf(mbuf, "%s removed from nickname open hush list.", who);
		        sendstatus(n, "Hush", mbuf);
			}
		 }
	      }
   	}
   }
   else
      if ((nlcount(*u_tab[n].pri_n_hushed) == 0) &&
	  (nlcount(*u_tab[n].pub_n_hushed) == 0) &&
	  (nlcount(*u_tab[n].pri_s_hushed) == 0) &&
	  (nlcount(*u_tab[n].pub_s_hushed) == 0))
		sendstatus(n, "Hush-List", "Empty List");
      else {
      if (nlcount(*u_tab[n].pri_n_hushed) > 0)
	 for (i = 0; i < nlcount(*u_tab[n].pri_n_hushed); i++) 
	     sendstatus(n, "Personal-Nick-Hushed", nlget(u_tab[n].pri_n_hushed));
      if (nlcount(*u_tab[n].pub_n_hushed) > 0)
	 for (i = 0; i < nlcount(*u_tab[n].pub_n_hushed); i++) 
	     sendstatus(n, "Open-Nick-Hushed", nlget(u_tab[n].pub_n_hushed));
      if (nlcount(*u_tab[n].pri_s_hushed) > 0)
	 for (i = 0; i < nlcount(*u_tab[n].pri_s_hushed); i++) 
	     sendstatus(n, "Personal-Site-Hushed", nlget(u_tab[n].pri_s_hushed));
      if (nlcount(*u_tab[n].pub_s_hushed) > 0)
	 for (i = 0; i < nlcount(*u_tab[n].pub_s_hushed); i++) 
	     sendstatus(n, "Open-Site-Hushed", nlget(u_tab[n].pub_s_hushed));
      }
return 0;
}

s_name(n, argc)
int n;
int argc;
{
	int len, i, j;
	int how_many;
	char new_name[MAX_NICKLEN+1];
	int ret;

       if (argc == 2) {
		/* constraints:
			will only use the first max_nicklen letters of
			a nickname.
			nickname must be at least 1 character long.
			to use the nickname admin, must have password
			set to TheKey
		*/

	/* only use so much of it, but at least some of it */
		len = strlen(fields[1]);
		if (len <= 0) {
			/* oops, too short */
                	senderror(n,"The nickname may not be null.");
                	return(-1);
		}

		how_many = (MAX_NICKLEN > len) ? len:MAX_NICKLEN;
		bzero(new_name, MAX_NICKLEN+1);
		strncpy(new_name, fields[1], how_many);
		filternickname(new_name);

	        /* make sure the nickname hasn't already been taken */

	        if ((find_user(new_name) >= 0) &&
		   (strcasecmp(new_name, u_tab[n].nickname) != 0)) {
			/* oops, someone already has this nick */
                	senderror(n,"Nickname already in use.");
                	return(-1);
        	}

        /* make sure they are allowed to use this nickname */
		if (strcasecmp(new_name,"admin") == 0) {
                if (strcmp(u_tab[n].password, TheKey) != 0) {
                        /* oops, password is wrong */
                        senderror(n,"Nickname already in use.");
                        return(-1);
                }
		}

		sprintf(mbuf,"%s changed nickname to %s",
			u_tab[n].nickname, new_name);
                s_status_group(1,0,n,"Name",mbuf);
		nickwritetime(n, 1);
		strcpy(u_tab[n].nickname, new_name);



		/* side-effects:
			inform folx in same group of change
			if they are registered, set their real name.
		*/
		sendstatus(n,"Name",mbuf);

        	/* check to see if we know this person */
		bzero(u_tab[n].realname, MAX_REALLEN+1);
	        ret = nicklookup(-n, u_tab[n].nickname);
	        if (ret == 0) {
	                /* we know this person */
	                strcpy(u_tab[n].realname, "registered");
			sends_cmdout(n, "Nick registered");
			nickwritetime(n, 0);
			for (i = 1; i <= MAX_GROUPS; i++)
			if ((g_tab[i].modtimeout > 0.0) &&
			  (strcmp(g_tab[i].missingmod, u_tab[n].nickname)==0)){
			  g_tab[i].modtimeout = 0;
			  g_tab[i].mod = n;
			  bzero(g_tab[i].missingmod, MAX_NICKLEN);
			  sprintf(mbuf, "%s is the active moderator again.",
				u_tab[n].nickname);
			  for (j = 1; j < MAX_REAL_USERS; j++)
				if ((strcasecmp(u_tab[j].group, g_tab[i].name)
				    == 0) && (j != n))
				    sendstatus(j, "Mod", mbuf);
			  sprintf(mbuf, "You are the moderator of group %s",
				g_tab[i].name);
			  sendstatus(n, "Mod", mbuf);
			}
	                if ((i = nickckmsg(n)) > 0)
                        	if (i == 1)
                                	sendstatus(n, 
					     "Message", "You have 1 message");
                        	else {
                                	sprintf(mbuf,"You have %d messages", i);
                                	sendstatus(n, "Message", mbuf);
                                }

/*			mdb("person is known"); */
	        } 
		else if (ret == -2) 
			/* we know this person but they're not validated */
		{
		sendstatus(n, "Register", "Send password to authenticate your nickname.");
		j = 0;
		for (i = 0; i < MAX_GROUPS; i++)
	    	if (strcasecmp(u_tab[n].nickname, g_tab[i].missingmod) == 0) {
			sprintf(mbuf, "You are moderator of group %s", g_tab[i].name);
			sendstatus(n, "Mod", mbuf);
			j++;
		    	}
		if (j == 1) sendstatus(n, "Mod", "You must register using /p <password> to regain mod of the above group.");
		if (j > 1) sendstatus(n, "Mod", "You must register using /p <password> to regain mod of the above groups.");
		}
		else {
	                /* we don't know this person */
	                bzero(u_tab[n].realname, MAX_REALLEN + 1);
			sendstatus(n, "No-Pass", "To register your nickname type /m server p mypassword");
/*			mdb("person is unknown"); */
		}
/*		mdb("finished checking on person"); */
        } else {
                mdb("name: wrong number of parz");
        }
}

s_notify(n, argc)
int n;
int argc;
{
	char *who, *flags;
	int quiet = -1;
	int mode = -1;
	int i;

	who = fields[1];
	while (who[0] == '-') {
		flags = getword(who);
		for (i = 1; i < strlen(flags); i++)
			switch(flags[i]) {
			case 'q':
				if (quiet == -1)
					quiet = 1;
				else {
					senderror(n, "Usage: notify {-q} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 'n':
				if (mode == -1)
					mode = 0;
				else {
					senderror(n, "Usage: notify {-q} {-n nickname | -s address}");
					return -1;
					}
				break;
			case 's':
				if (mode == -1)
					mode = 1;
				else {
					senderror(n, "Usage: notify {-q} {-n nickname | -s address}");
					return -1;
					}
				break;
			default:
				senderror(n, "Usage: notify {-q} {-n nickname | -s address}");
				return -1;
				break;
			}
		who = get_tail(who);
		}
	if (mode == -1) mode = 0;
	if (quiet == -1) quiet = 0;
	ucaseit(who);

	if (strlen(who) == 0)
	   if ((nlcount(*u_tab[n].n_notifies) == 0) &&
	      (nlcount(*u_tab[n].s_notifies) == 0))
		   sendstatus(n, "Notify-List", "Empty List");
	   else {
		if (nlcount(*u_tab[n].n_notifies) > 0)
		   for (i = 0; i < nlcount(*u_tab[n].n_notifies); i++)
		       sendstatus(n, "Notify-Nickname", nlget(u_tab[n].n_notifies));
		if (nlcount(*u_tab[n].s_notifies) > 0)
		   for (i = 0; i < nlcount(*u_tab[n].s_notifies); i++)
		       sendstatus(n, "Notify-Site", nlget(u_tab[n].s_notifies));
		}
	else 
	   if (mode == 1)
	      if (nlpresent(who, *u_tab[n].s_notifies) == 0) {
		if (nlcount(*u_tab[n].s_notifies) == MAX_NOTIFIES) {
			senderror(n, "Max number of site notifies reached.");
			return -1;
			}
		nlput(u_tab[n].s_notifies, who);
		if (quiet == 0) {
			sprintf(mbuf, "%s added to site notify list.", who);
			sendstatus(n, "Notify", mbuf);
			}
		}
	      else {
		nldelete(u_tab[n].s_notifies, who);
		if (quiet == 0) {
			sprintf(mbuf, "%s removed from site notify list.", who);
			sendstatus(n, "Notify", mbuf);
			}
		}
	   else 
	      if (nlpresent(who, *u_tab[n].n_notifies) == 0) {
		if (nlcount(*u_tab[n].n_notifies) == MAX_NOTIFIES) {
			senderror(n,"Max number of nickname notifies reached.");
			return -1;
			}
		nlput(u_tab[n].n_notifies, who);
		if (quiet == 0) {
			sprintf(mbuf, "%s added to nickname notify list.", who);
			sendstatus(n, "Notify", mbuf);
			if ((i = find_user(who)) > 0) {
			   sprintf(mbuf, "%s is logged in now.", u_tab[i].nickname);
			   sendstatus(n, "Notify", mbuf);
			   }
			}
		}
	      else {
		nldelete(u_tab[n].n_notifies, who);
		if (quiet == 0) {
			sprintf(mbuf, "%s removed from nickname notify list.", who);
			sendstatus(n, "Notify", mbuf);
			}
		}
return 0;
}

s_echoback(n, argc)
int n;
int argc;
{
       if (argc == 2) {
		if(strcasecmp(fields[1],"off") == 0) {
			u_tab[n].echoback = 0; /* off */
			sendstatus(n, "Echo","Echoback off");	
		} else if (strcasecmp(fields[1],"on") == 0) {
			u_tab[n].echoback = 1; /* on */
			sendstatus(n,"Echo","Echoback on");	
		} else if (strcasecmp(fields[1],"verbose") == 0) {
			u_tab[n].echoback = 2; /* verbose */
			sendstatus(n,"Echo","Echoback on verbose");
		} else {
			senderror(n,"Echoback: needs on/off/verbose");
		}
        } else {
                mdb("echoback: wrong number of parz");
        }
}
