/* Copyright (c) 1991 by John Atwood deVries II. */
/* For copying and distribution information, see the file COPYING. */

/* send various messages to the client */

#include "server.h"
#include "externs.h"
#include "protocol.h"
#include "version.h"
#include <strings.h>
#include <errno.h>

extern long time();
extern char * getremotename();

/* send normal open group message to the client */
sendopen(n, i, txt)
int n;
int i;
char *txt;
{
	filtertext(txt);
	/* construct open message */
	sprintf(pbuf,"%c%s\001%s", M_OPEN, u_tab[n].nickname, txt);
	doSend(n, i);
}

/* send an exit message to the client -- makes the client disconnect */
sendexit(n)
int n;
{
	/* construct exit message */
	sprintf(pbuf, "%c",M_EXIT);
	doSend(-1, n);
}

/* send a ping */
sendping(n)
int n;
{
	/* construct pint message */
	sprintf(pbuf, "%c",M_PING);
	doSend(-1, n);
}


/* send an error message to the client */
senderror(n, error_string)
int n;
char *error_string;
{
	filtertext(error_string);
        sprintf(pbuf, "%c%s", M_ERROR, error_string);
	doSend(-1, n);
}

/* send a status message to the client */
sendstatus(n, class_string, message_string)
int n;
char *class_string;
char *message_string;
{
	filtertext(class_string);
	filtertext(message_string);
        sprintf(pbuf, "%c%s\001%s", M_STATUS, class_string, message_string);
	doSend(-1, n);
}

/* send "end of command" to the client */
send_cmdend(n, output_string)
int n;
char *output_string;
{
	filtertext(output_string);
      	sprintf(pbuf, "%c%s\001\001%s", M_CMDOUT, "ec", output_string);
	doSend(-1, n);
}

/* send simple command output message to the client */
sends_cmdout(n, output_string)
int n;
char *output_string;
{
	filtertext(output_string);
      	sprintf(pbuf, "%c%s\001%s", M_CMDOUT, "co", output_string);
	doSend(-1, n);
}

/* send personal message to a client */
sendperson(from, to, message_string)
int from;
int to;
char *message_string;
{
	char one[255], two[MAX_NICKLEN+1];

	filtertext(message_string);
        sprintf(one, "%s@%s", u_tab[from].loginid, u_tab[from].nodeid);
        ucaseit(one);
        strncpy(two, u_tab[from].nickname, MAX_NICKLEN);
        ucaseit(two);

	if ((!nlmatch(one, *u_tab[to].pri_s_hushed)) &&
           (!nlmatch(two, *u_tab[to].pri_n_hushed))) {
        	sprintf(pbuf, "%c%s\001%s", M_PERSONAL, u_tab[from].nickname,
						 message_string);
		doSend(from, to);
		}
	else
		sendstatus(from, "Bounce", "Message did not go through");
}

/* send beep message to a client */
sendbeep(from, to)
int from;
int to;
{
        sprintf(pbuf, "%c%s", M_BEEP, u_tab[from].nickname);
	doSend(from, to);
}

s_new_user(n)
int n;  /* fd of their socket */
{
	char *cp;

        /* construct proto(col) message */
        sprintf(pbuf, "%c%d\001%s\001%s",M_PROTO,PROTO_LEVEL,
           thishost, VERSION);
	doSend(-1, n);
        cp = getremotename(n);
        if (cp == NULL) {
		sprintf(mbuf, "[CONNECT] %d", n);
		mdb(mbuf);
		return;
		}
        if (strlen(cp) == 0) {
		sprintf(mbuf, "[CONNECT] %d", n);
		mdb(mbuf);
		return;
		}
        if(strcasecmp(SHORT_HOSTNAME, cp) == 0) {
                cp = FQDN;
        }
	sprintf(mbuf, "[CONNECT] %d: %s", n, cp);
	bzero(u_tab[n].nodeid, MAX_NODELEN+1);
	strncpy(u_tab[n].nodeid, cp, MAX_NODELEN);
	mdb(mbuf);
}

send_loginok(n)
int n;
{
	/* construct loginok message */
	sprintf(pbuf, "%c",M_LOGINOK);
	doSend(-1, n);
}

/* send an important message to the client */
sendimport(n, class_string, output_string)
int n;
char *class_string;
char *output_string;
{
	filtertext(class_string);
	filtertext(output_string);
        sprintf(pbuf, "%c%s\001%s", M_IMPORTANT, class_string,
		output_string);
	doSend(-1, n);
}

doSend(from,to)
int from;
int to;
{
	int ret;
	char line[256];
	char tbuf[256];

	if(to < 0) {
		sprintf(tbuf, "[ERROR] Attempted to send to negative fd: %d",
			to);
		mdb(tbuf);
		return -1;
		}
	if(to >= MAX_REAL_USERS) {
		*pp = ' ';
		switch (*(pp+1)) {
		case M_BEEP: /* someone beeped us */
			autoBeep(from);
			break;
		case M_PERSONAL: /* someone sent us a message */
			/* autoCommand(from, 1); */
			split(pp);
			bzero(line, 256);
			sprintf(line, "%s\001%s", getword(fields[1]), 
				get_tail(fields[1]));
			cmdmsg(from, line);
			break;
		default: /* do nothing -- ignore */
			break;
		}
	} else {
		if (strlen(pbuf) > 254) {
			mdb ("[ERROR] doSend: pbuf too large:");
			sprintf (mbuf, "[ERROR] %s", pbuf);
			mdb (mbuf);
			sprintf (mbuf, "Cannot transmit packet: too large");
			senderror(from, mbuf);
			return -1;
			}
		if (S_kill[to] > 0) return -1;
		setlen;
		if ((ret = _sendpacket(to,pp)) < 0) {
			sprintf(tbuf, "[ERROR] doSend: %d: %s (%d)",
				to, sys_errlist[errno], ret);
			mdb(tbuf);
			if (ret == -2) {
				/* bad news! */
				S_kill[to]++;
				/* need to clear the notifies of this user,
				   lest he try to someone else who's dead
				   who has this guy in his notify. Endless
				   loop! */
				/* Don't know if it's still needed, but it
				   doesn't hurt */
				nlclear(u_tab[to].n_notifies);
				nlclear(u_tab[to].s_notifies);
				}
			}
	}
}

autoBeep(n)
int n;
{
	sendperson(NICKSERV, n, "Beep yerself!");
	u_tab[NICKSERV].t_recv = time(NULL);
}

/*
	Copyright (c) 1991 by Keith Graham
	Modifications Copyright (c) 1991 by John Atwood deVries II

	"to" is the destination socket..
*/

void
user_wline(to,mod,nick,idle,resp,login,user,site,name)
int to;
char *mod,*nick;
int idle, resp, login;
char *user, *site, *name;
{ 
sprintf(pbuf,"%cwl\001%s\001%s\001%ld\001%ld\001%ld\001%s\001%s\001%s\000",
			M_CMDOUT,mod,nick,idle,resp,login,
				user,site,name);
doSend(-1, to);
}

void
user_wgroupline(to,group,topic,c,d)
int 	to;
char	*group;
char	*topic;
{ 
sprintf(pbuf,"%cwg\001%s\001%s\001%s\001%s\000",M_CMDOUT,group,topic,c,d);
doSend(-1, to);
}

void
user_whead(to)
int to;
{ 
sprintf(pbuf,"%cwh\000",M_CMDOUT);
doSend(-1, to);
}
